@@ -1,4 +1,5 @@
-extern crate num_bigint;
+extern crate ibig;
+extern crate num_traits;
 extern crate sesdiff;
 
 use std::fs::File;
@@ -29,10 +30,16 @@ pub use crate::confusables::*;
 pub use crate::cache::*;
 
 
+/// The VariantModel is the most high-level model of analiticcl, it holds
+/// all data required for variant matching.
 pub struct VariantModel {
+    /// Maps Vocabulary IDs to their textual strings and other related properties
     pub decoder: VocabDecoder,
+
+    /// Map strings to vocabulary IDs
     pub encoder: VocabEncoder,
 
+    /// Defines the alphabet used for the variant model
     pub alphabet: Alphabet,
 
     ///The main index, mapping anagrams to instances
@@ -56,15 +63,21 @@ pub struct VariantModel {
     /// items for provenance reasons
     pub lexicons: Vec<String>,
 
+    /// Holds weighted confusable recipes that can be used in scoring and ranking
     pub confusables: Vec<Confusable>,
 
     ///Process confusables before pruning by max_matches
     pub confusables_before_pruning: bool,
 
+    /// Groups clusters of variants (either from explicitly loaded variant files or in a later
+    /// stage perhaps also computed)
+    pub variantclusters: VariantClusterMap,
+
     pub debug: bool
 }
 
 impl VariantModel {
+    /// Instantiate a new variant model
     pub fn new(alphabet_file: &str, weights: Weights, debug: bool) -> VariantModel {
         let mut model = VariantModel {
             alphabet: Vec::new(),
@@ -74,16 +87,19 @@ impl VariantModel {
             sortedindex: BTreeMap::new(),
             have_freq: false,
             freq_sum: 0,
-            weights: weights,
+            weights,
             lexicons: Vec::new(),
             confusables: Vec::new(),
             confusables_before_pruning: false,
-            debug: debug,
+            variantclusters: HashMap::new(),
+            debug,
         };
         model.read_alphabet(alphabet_file).expect("Error loading alphabet file");
         model
     }
 
+    /// Instantiate a new variant model, explicitly passing an alphabet rather than loading one
+    /// from file.
     pub fn new_with_alphabet(alphabet: Alphabet, weights: Weights, debug: bool) -> VariantModel {
         VariantModel {
             alphabet: alphabet,
@@ -93,23 +109,29 @@ impl VariantModel {
             sortedindex: BTreeMap::new(),
             have_freq: false,
             freq_sum: 0,
-            weights: weights,
+            weights,
             lexicons: Vec::new(),
             confusables: Vec::new(),
             confusables_before_pruning: false,
-            debug: debug,
+            variantclusters: HashMap::new(),
+            debug,
         }
     }
 
+    /// Configure the model to match against known confusables prior to pruning on maximum weight.
+    /// This may lead to better results but may have a significant performance impact.
     pub fn set_confusables_before_pruning(&mut self) {
         self.confusables_before_pruning = true;
     }
 
+    /// Returns the size of the alphabet, this is typically +1 longer than the actual alphabet file
+    /// as it includes the UNKNOWN symbol.
     pub fn alphabet_size(&self) -> CharIndexType {
         self.alphabet.len() as CharIndexType + 1 //+1 for UNK
     }
 
-    pub fn get_or_create_node<'a,'b>(&'a mut self, anahash: &'b AnaValue) -> &'a mut AnaIndexNode {
+    /// Get an item from the index or insert it if it doesn't exist yet
+    pub fn get_or_create_index<'a,'b>(&'a mut self, anahash: &'b AnaValue) -> &'a mut AnaIndexNode {
             if self.contains_key(anahash) {
                 self.index.get_mut(anahash).expect("get_mut on node after check")
             } else {
@@ -121,6 +143,8 @@ impl VariantModel {
             }
     }
 
+    /// Build the anagram index (and secondary index) so the model
+    /// is ready for variant matching
     pub fn build(&mut self) {
         if !self.have_freq {
             self.weights.freq = 0.0
@@ -145,7 +169,7 @@ impl VariantModel {
         eprintln!("Adding all instances to the index...");
         for (anahash, id) in tmp_hashes {
             //add it to the index
-            let node = self.get_or_create_node(&anahash);
+            let node = self.get_or_create_index(&anahash);
             node.instances.push(id);
         }
         eprintln!(" - Found {} anagrams", self.index.len() );
@@ -169,6 +193,7 @@ impl VariantModel {
         }
     }
 
+    /// Tests if the anagram value exists in the index
     pub fn contains_key(&self, key: &AnaValue) -> bool {
         self.index.contains_key(key)
     }
@@ -227,7 +252,13 @@ impl VariantModel {
         for line in f_buffer.lines() {
             if let Ok(line) = line {
                 if !line.is_empty() {
-                    self.alphabet.push(line.split("\t").map(|x| x.to_owned()).collect());
+                    self.alphabet.push(line.split("\t").map(|x|
+                            match x {
+                                "\\s" => " ".to_owned(),
+                                "\\t" => "\t".to_owned(),
+                                "\\n" => "\n".to_owned(),
+                                _ => x.to_owned()
+                            }).collect());
                 }
 
             }
@@ -269,6 +300,7 @@ impl VariantModel {
         Ok(())
     }
 
+    /// Add a confusable
     pub fn add_to_confusables(&mut self, editscript: &str, weight: f64) -> Result<(), std::io::Error> {
         let confusable = Confusable::new(editscript, weight)?;
         self.confusables.push(confusable);
@@ -282,6 +314,7 @@ impl VariantModel {
         if self.debug {
             eprintln!("Reading vocabulary from {}...", filename);
         }
+        let beginlen = self.decoder.len();
         let f = File::open(filename)?;
         let f_buffer = BufReader::new(f);
         for line in f_buffer.lines() {
@@ -300,13 +333,107 @@ impl VariantModel {
             }
         }
         if self.debug {
-            eprintln!(" - Read vocabulary of size {}", self.decoder.len());
+            eprintln!(" - Read vocabulary of size {}", self.decoder.len() - beginlen);
         }
         self.lexicons.push(filename.to_string());
         Ok(())
     }
 
-    pub fn add_to_vocabulary(&mut self, text: &str, frequency: Option<u32>, lexicon_weight: Option<f32>, lexicon_index: u8) {
+    ///Read a variants list of equally weighted variants from a TSV file
+    ///Each line simply contains tab-separated variants and all entries on a single line are
+    ///considered variants. Consumed much less memory than weighted variants.
+    pub fn read_variants(&mut self, filename: &str, lexicon_weight: f32) -> Result<(), std::io::Error> {
+        if self.debug {
+            eprintln!("Reading variants from {}...", filename);
+        }
+        let beginlen = self.variantclusters.len();
+        let f = File::open(filename)?;
+        let f_buffer = BufReader::new(f);
+        for line in f_buffer.lines() {
+            if let Ok(line) = line {
+                if !line.is_empty() {
+                    let variants: Vec<&str> = line.split("\t").collect();
+                    let mut ids: Vec<VocabId> = Vec::new();
+                    let clusterid = self.variantclusters.len() as VariantClusterId;
+                    for variant in variants.iter() {
+                        //all variants by definition are added to the combined lexicon
+                        let variantid = self.add_to_vocabulary(variant, None, Some(lexicon_weight), self.lexicons.len() as u8);
+                        ids.push(variantid);
+                        if let Some(vocabvalue) = self.decoder.get_mut(variantid as usize) {
+                            let variantref = VariantReference::VariantCluster(clusterid);
+                            if vocabvalue.variants.is_none() {
+                                vocabvalue.variants = Some(vec!(variantref));
+                            } else if let Some(variantrefs) = vocabvalue.variants.as_mut() {
+                                if !variantrefs.contains(&variantref) {
+                                    variantrefs.push(variantref);
+                                }
+                            }
+                        }
+                    }
+                    self.variantclusters.insert(clusterid, ids);
+                }
+            }
+        }
+        if self.debug {
+            eprintln!(" - Read variants list, added {} new variant clusters", self.variantclusters.len() - beginlen);
+        }
+        self.lexicons.push(filename.to_string());
+        Ok(())
+    }
+
+    ///Read a weighted variant list from a TSV file. Contains a canonical/reference form in the
+    ///first column, and variants with score (two columns) in the following columns. Consumes much more
+    ///memory than equally weighted variants.
+    pub fn read_weighted_variants(&mut self, filename: &str, lexicon_weight: f32, intermediate: bool) -> Result<(), std::io::Error> {
+        if self.debug {
+            eprintln!("Reading variants from {}...", filename);
+        }
+        let mut count = 0;
+        let f = File::open(filename)?;
+        let f_buffer = BufReader::new(f);
+        for line in f_buffer.lines() {
+            if let Ok(line) = line {
+                if !line.is_empty() {
+                    let fields: Vec<&str> = line.split("\t").collect();
+
+                    let reference = fields.get(0).expect("first item");
+                    let ref_id = self.add_to_vocabulary(reference, None, Some(lexicon_weight), self.lexicons.len() as u8);
+                    let mut iter = fields.iter();
+
+                    while let (Some(variant), Some(score)) = (iter.next(), iter.next()) {
+                        let score = score.parse::<f64>().expect("Scores must be a floating point value");
+                        //all variants by definition are added to the lexicon
+                        let variantid = self.add_to_vocabulary(variant, None, Some(lexicon_weight), self.lexicons.len() as u8);
+                        if variantid != ref_id {
+                            if let Some(vocabvalue) = self.decoder.get_mut(ref_id as usize) {
+                                let variantref = VariantReference::WeightedVariant((variantid,score) );
+                                vocabvalue.intermediate = intermediate;
+                                if vocabvalue.variants.is_none() {
+                                    vocabvalue.variants = Some(vec!(variantref));
+                                    count += 1;
+                                } else if let Some(variantrefs) = vocabvalue.variants.as_mut() {
+                                    if !variantrefs.contains(&variantref) {
+                                        variantrefs.push(variantref);
+                                        count += 1;
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        if self.debug {
+            eprintln!(" - Read weighted variants list, added {} references", count);
+        }
+        self.lexicons.push(filename.to_string());
+        Ok(())
+    }
+
+
+
+    /// Adds an entry in the vocabulary
+    pub fn add_to_vocabulary(&mut self, text: &str, frequency: Option<u32>, lexicon_weight: Option<f32>, lexicon_index: u8) -> VocabId {
         let frequency = frequency.unwrap_or(1);
         let lexicon_weight = lexicon_weight.unwrap_or(1.0);
         if self.debug {
@@ -319,6 +446,7 @@ impl VariantModel {
                 item.lexweight = lexicon_weight;
                 item.lexindex = lexicon_index;
             }
+            *vocab_id
         } else {
             //item is new
             self.encoder.insert(text.to_string(), self.decoder.len() as u64);
@@ -329,7 +457,10 @@ impl VariantModel {
                 tokencount: text.chars().filter(|c| *c == ' ').count() as u8 + 1,
                 lexweight: lexicon_weight,
                 lexindex: lexicon_index,
+                variants: None,
+                intermediate: false,
             });
+            self.decoder.len() as VocabId - 1
         }
     }
 
@@ -569,9 +700,43 @@ impl VariantModel {
                                 suffixlen: if self.weights.suffix > 0.0 { common_suffix_length(querystring, &vocabitem.norm) } else { 0 },
                                 freq: if self.weights.freq > 0.0 { vocabitem.frequency } else { 0 },
                                 lex: if self.weights.lex > 0.0 { vocabitem.lexweight } else { 0.0 },
-                                samecase: if self.weights.case > 0.0 { vocabitem.text.chars().next().expect("first char").is_lowercase() == query.chars().next().expect("first char").is_lowercase() } else { true }
+                                samecase: if self.weights.case > 0.0 { vocabitem.text.chars().next().expect("first char").is_lowercase() == query.chars().next().expect("first char").is_lowercase() } else { true },
+                                prescore: None,
                             };
-                            found_instances.push((*vocab_id,distance));
+                            //match will be added to found_instances at the end of the block (we
+                            //need to borrow the distance for a bit still)
+
+                            //Does this vocabulary item make explicit references to variants?
+                            //If so, we add those too. This is only the case if the user loaded
+                            //variantlists/error lists.
+                            if let Some(variantrefs) = &vocabitem.variants {
+                                for variantref in variantrefs.iter() {
+                                    match variantref {
+                                        VariantReference::VariantCluster(cluster_id) => {
+                                            if let Some(variants) = self.variantclusters.get(cluster_id) {
+                                                //add all variants in the cluster
+                                                for variant_id in variants.iter() {
+                                                    //we clone do not recompute the distance to the
+                                                    //variant, all variants are considered of
+                                                    //equal-weight, we use the originally computed
+                                                    //distance:
+                                                    found_instances.push((*variant_id, distance.clone()));
+                                                }
+                                            }
+                                        },
+                                        VariantReference::WeightedVariant((vocab_id, score)) => {
+                                            let mut variantdistance = distance.clone();
+                                            variantdistance.prescore = Some(*score);
+                                            found_instances.push((*vocab_id,variantdistance));
+                                        }
+                                    }
+                                }
+                            }
+
+                            //add the original match
+                            if !vocabitem.intermediate {
+                                found_instances.push((*vocab_id,distance));
+                            }
                         } else {
                             pruned_instances += 1;
                         }
@@ -648,7 +813,7 @@ impl VariantModel {
                 } else {
                     1.0
                 };
-                let score = (
+                let mut score = (
                     self.weights.ld * distance_score +
                     self.weights.freq * freq_score +  //weight will be 0 if there are no frequencies
                     self.weights.lcs * lcs_score +
@@ -657,6 +822,10 @@ impl VariantModel {
                     self.weights.lex * vocabitem.lexweight as f64 +
                     if distance.samecase { self.weights.case } else { 0.0 }
                 ) / weights_sum;
+                if let Some(prescore) = distance.prescore {
+                    //variant is already pre-scored (it comes from an explicit weighted variant list), take the prescore into consideration:
+                    score = (score + prescore) / 2.0;
+                }
                 if score.is_nan() {
                     //should never happen
                     panic!("Invalid score (NaN) computed for variant={}, distance={:?}, score={}", vocabitem.text, distance, score);
@@ -736,6 +905,7 @@ impl VariantModel {
         results
     }
 
+    /// Rescores the scored variants by testing against known confusables
     pub fn rescore_confusables(&self, results: &mut Vec<(VocabId,f64)>, input: &str) {
         if self.debug {
             eprintln!("   (rescoring with confusable weights)");
